/*
  Color.h - Library for controlling RGB objects
  Created by Jesse R. Castro 2013-02-03
  Reworked by Maxime Fouillat in 2022-06
    Switch to RGB-based instead of HSV-based
    Added two-way redundancy to and from RGB, fixes empty RGB values
    Converted properties to interger and byte logic instead of floating point for lesser memory usage
    Fixed operator overloading for RGB compatibility
    Reorganized public and private parts of the class
*/
#ifndef Color_h
#define Color_h

#include "Arduino.h"

class Color {
  public:	
    //constructor
      Color(int pRed, int pGreen, int pBlue);
      //Color(int pHue, int pSaturation, int pValue);
      Color(const Color& pColor);
      Color();
    //destructor
    //public vars
      int red;
      int green;
      int blue;
      int hue;
      int saturation;
      int value;
    //setters

    //getters
      int getHue() const;
      int getSaturation() const;
      int getValue() const;
      int getRed();
      int getBlue();
      int getGreen();
    //action methods
      void turnOff();	
    //operator overloads
      Color& operator=(const Color &pColor);
      bool operator==(const Color &pColor) const;
      bool operator!=(const Color &pColor) const;
  private:
    //private vars
      void setRed(int pRed);
      void setGreen(int pGreen);
      void setBlue(int pBlue);
      void setRGB(int pRed, int pBlue, int pGreen);
    //setters
      void setHue(int pHue);
      void setSaturation(int pSaturation);
      void setValue(int pValue);
      void setHSV(int pHue, int pSaturation, int pValue);
    //getters
    
    //action methods
    
    //utilities
      void rgb2hsv();
      void hsv2rgb();
      float getMin();
      float getMax();
      int floor(float target);
};

#endif